package com.hzs.member.statis.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.hzs.common.domain.member.statis.CuBonusVertexStatis;
import com.hzs.member.statis.vo.*;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 奖金拨比零线统计 Mapper 接口
 */
public interface CuBonusVertexStatisMapper extends BaseMapper<CuBonusVertexStatis> {

    /**
     * @description: 奖金拨比列表
     **/
    List<CuBonusVertexStatisVO> queryCuBonusStatis(CuBonusVertexStatisVO cuBonusVertexStatisVO);

    /**
     * @description: 查询顶点奖金拨比汇总
     **/
    List<CuBonusVertexStaticTotalVO> queryCuBonusStatisTotal(CuBonusVertexStaticTotalVO cuBonusVertexStaticTotalVO);

    /**
     * @description: 顶点奖金拨比汇总查询总和
     **/
    CuBonusVertexStaticTotalVO queryCuBonusStatisSum(CuBonusVertexStaticTotalVO cuBonusVertexStaticTotalVO);

    /**
     * @description: 查询时间内奖金拨比顶点会员
     **/
    List<CuBonusVertexStaticTotalVO> queryCuBonusVertex(CuBonusVertexStaticTotalVO cuBonusVertexStaticTotalVO);

    /**
     * 查询顶点拓展封顶人数
     **/
    CuBonusVertexStaticTotalVO queryOrgTopNumber(CuBonusVertexStatisVO cuBonusVertexStatisVO);

    /**
     * 查询顶点拓展平均碰次和拓展最高碰次
     **/
    CuBonusVertexStaticTotalVO queryTouch(CuBonusVertexStatisVO cuBonusVertexStatisVO);

    /**
     * @description: 奖金拨比列表统计小计（最后一行）
     **/
    CuBonusVertexStatisVO queryCuBonusSumStatis(CuBonusVertexStatisVO CuBonusVertexStatisVO);

    /**
     * @description: 查询小于当天全部已支付或撤单订单信息
     **/
    List<CuBonusVertexStatisVO> querySaOrderByLessDay(CuBonusVertexStatisVO CuBonusVertexStatisVO);

    /**
     * @description: 7天重算查询订单
     **/
    List<CuBonusVertexStatisVO> queryAnewSaOrderByLessDay(CuBonusVertexStatisVO CuBonusVertexStatisVO);


    /**
     * @description: 删除奖金拨比统计表数据
     **/
    Boolean delStatis(@Param("startDate") String startDate,
                      @Param("endDate") String endDate,
                      @Param("pkVertexList") List<Integer> pkVertexList);

    /**
     * @description: 查询奖金扣项配置参数
     **/
    List<CuBonusVertexStatisVO> bonusDeductlist(CuBonusVertexStatisVO CuBonusVertexStatisVO);

    /**
     * @description: 查询中国奖金
     **/
    List<CuBonusVertexStatisVO> queryChinaCountryBonus(CuBonusVertexStatisVO CuBonusVertexStatisVO);

    /**
     * @description: 查询7天重算奖金查询
     **/
    List<CuBonusVertexStatisVO> queryVertexChinaCountryBonus(CuBonusVertexStatisVO CuBonusVertexStatisVO);

    /**
     * @description: 查询生效国外国家列表
     **/
    List<CuBonusVertexStatisVO> countrylist(CuBonusVertexStatisVO CuBonusVertexStatisVO);

    /**
     * @description: 查询有效顶点
     **/
    List<CuBonusVertexStatisVO> vertexList(@Param("pkVertex") Integer pkVertex);

    /**
     * @description: 查询当天团队统计数据
     **/
    List<CuBonusVertexStatis> getCuBonusStatis(CuBonusVertexStatis CuBonusVertexStatis);

}
